@@ -1,6 +1,8 @@
 from datanator_query_python.util import mongo_util, chem_util, file_util
 from . import query_nosql
 import json
+import re
+
 
 class QuerySabioOld(query_nosql.DataQuery):
     '''Queries specific to sabio_rk collection
@@ -8,18 +10,19 @@ class QuerySabioOld(query_nosql.DataQuery):
 
     def __init__(self, cache_dirname=None, MongoDB=None, replicaSet=None, db='datanator',
                  collection_str='sabio_rk_old', verbose=False, max_entries=float('inf'), username=None,
-                 password=None, authSource='admin'):
+                 password=None, authSource='admin', readPreference='nearest'):
         self.max_entries = max_entries
-        super(query_nosql.DataQuery, self).__init__(cache_dirname=cache_dirname, MongoDB=MongoDB,
-                                        replicaSet=replicaSet, db=db,
-                                        verbose=verbose, max_entries=max_entries, username=username,
-                                        password=password, authSource=authSource)
+        super().__init__(cache_dirname=cache_dirname, MongoDB=MongoDB,
+                        replicaSet=replicaSet, db=db,
+                        verbose=verbose, max_entries=max_entries, username=username,
+                        password=password, authSource=authSource, readPreference=readPreference)
         self.chem_manager = chem_util.ChemUtil()
         self.file_manager = file_util.FileUtil()
         self.client, self.db_obj, self.collection = self.con_db(collection_str)
+        self.collection_str = collection_str
 
     def get_kinlaw_by_environment(self, taxon=None, taxon_wildtype=None, ph_range=None, temp_range=None,
-                          name_space=None, observed_type=None, projection={'_id': 0}):
+                          name_space=None, param_type=None, projection={'_id': 0}):
         """get kinlaw info based on experimental conditions
         
         Args:
@@ -28,7 +31,7 @@ class QuerySabioOld(query_nosql.DataQuery):
             ph_range (:obj:`list`, optional): range of pH
             temp_range (:obj:`list`, optional): range of temperature
             name_space (:obj:`dict`, optional): cross_reference key/value pair, i.e. {'ec-code': '3.4.21.62'}
-            observed_type (:obj:`list`, optional): possible values for parameters.observed_type
+            param_type (:obj:`list`, optional): possible values for parameters.type
             projection (:obj:`dict`, optional): mongodb query result projection
 
         Returns:
@@ -50,8 +53,8 @@ class QuerySabioOld(query_nosql.DataQuery):
             key = list(name_space.keys())[0]
             val = list(name_space.values())[0]
             all_constraints.append({"resource": {'$elemMatch': {'namespace': key, 'id': val}}})
-        if observed_type:
-            all_constraints.append({'parameter': {'$elemMatch': {'sbo_type': {'$in': observed_type}}}})
+        if param_type:
+            all_constraints.append({'parameter': {'$elemMatch': {'type': {'$in': param_type}}}})
 
         query = {'$and': all_constraints}
         docs = self.collection.find(filter=query, projection=projection)
@@ -74,26 +77,75 @@ class QuerySabioOld(query_nosql.DataQuery):
         count = self.collection.count_documents(query)
         return docs, count
 
-    def get_kinlawid_by_rxn(self, substrates, products):
+    def get_kinlawid_by_rxn(self, substrates, products, dof=0):
         ''' Find the kinlaw_id defined in sabio_rk using 
-            rxn participants' inchi string
+            rxn participants' inchikey
 
             Args:
-                substrates: list of substrates' inchi
-                products: list of products' inchi
+                substrates (:obj:`list`): list of substrates' inchikey
+                products (:obj:`list`): list of products' inchikey
+                dof (:obj:`int`, optional): degree of freedom allowed (number of parts of
+                                  inchikey to truncate); the default is 0 
 
             Return:
                 rxns: list of kinlaw_ids that satisfy the condition
                 [id0, id1, id2,...,  ]
         '''
         result = []
-        substrate = 'reaction_participant.substrate.substrate_structure.InChI_Key'
-        product = 'reaction_participant.product.product_structure.InChI_Key'
+        substrate = 'reaction_participant.substrate_aggregate'
+        product = 'reaction_participant.product_aggregate'
         projection = {'kinlaw_id': 1, '_id': 0}
-        constraint_0 = {substrate: {'$in': substrates}}
-        constraint_1 = {product: {'$in': products}}
+        if dof == 0:
+            substrates = substrates
+            products = products
+        elif dof == 1:
+            substrates = [re.compile('^' + x[:-2]) for x in substrates]
+            products = [re.compile('^' + x[:-2]) for x in products]
+        else:
+            substrates = [re.compile('^' + x[:14]) for x in substrates]
+            products = [re.compile('^' + x[:14]) for x in products]
+
+        constraint_0 = {substrate: {'$all': substrates}}
+        constraint_1 = {product: {'$all': products}}
         query = {'$and': [constraint_0, constraint_1]}
         docs = self.collection.find(filter=query, projection=projection)
         for doc in docs:
             result.append(doc['kinlaw_id'])
-        return result
\ No newline at end of file
+        return result
+
+    def get_kinlaw_by_rxn(self, substrates, products, dof=0,
+                          projection={'kinlaw_id': 1, '_id': 0}):
+        ''' Find the kinlaw_id defined in sabio_rk using 
+            rxn participants' inchikey
+
+            Args:
+                substrates (:obj:`list`): list of substrates' inchikey
+                products (:obj:`list`): list of products' inchikey
+                dof (:obj:`int`, optional): degree of freedom allowed (number of parts of
+                                  inchikey to truncate); the default is 0
+                projection (:obj:`dict`): pymongo query projection 
+
+            Return:
+                (:obj:`list` of :obj:`dict`): list of kinlaws that satisfy the condition
+        '''
+        result = []
+        substrate = 'reaction_participant.substrate_aggregate'
+        product = 'reaction_participant.product_aggregate'
+        if dof == 0:
+            substrates = substrates
+            products = products
+        elif dof == 1:
+            substrates = [re.compile('^' + x[:-2]) for x in substrates]
+            products = [re.compile('^' + x[:-2]) for x in products]
+        else:
+            substrates = [re.compile('^' + x[:14]) for x in substrates]
+            products = [re.compile('^' + x[:14]) for x in products]
+
+        constraint_0 = {substrate: {'$all': substrates}}
+        constraint_1 = {product: {'$all': products}}
+        query = {'$and': [constraint_0, constraint_1]}
+        docs = self.collection.find(filter=query, projection=projection)
+        for doc in docs:
+            result.append(doc)
+        count = self.collection.count_documents(query)
+        return count, result
\ No newline at end of file
